source("modules/Anomalies/tmp/CST_Anomaly.R")

# Compute the hcst, obs and fcst anomalies with or without cross-validation
# and return them, along with the hcst and obs climatologies.

compute_anomalies <- function(recipe, data) {

  if (is.null(recipe$Analysis$Workflow$Anomalies$compute)) {
    error(recipe$Run$logger, 
	  paste("The anomaly module has been called, but the element",
		"'Workflow:Anomalies:compute' is missing from the recipe."))
    stop()
  }

  if (recipe$Analysis$Workflow$Anomalies$compute) {
    if (recipe$Analysis$Workflow$Anomalies$cross_validation) {
      cross <- TRUE
      cross_msg <- "with"
    } else {
      cross <- FALSE
      cross_msg <- "without"
    }
    original_dims <- dim(data$hcst$data)

    # Compute anomalies
    anom <- CST_Anomaly(data$hcst, data$obs,
			cross = cross,
			memb = TRUE,
			memb_dim = 'ensemble',
			dim_anom = 'syear',
			dat_dim = c('dat', 'ensemble'),
			ftime_dim = 'time',
			ncores = recipe$Analysis$ncores)
    # Reorder dims
    anom$exp$data <- Reorder(anom$exp$data, names(original_dims))
    anom$obs$data <- Reorder(anom$obs$data, names(original_dims))

    # Save full fields
    hcst_fullvalue <- data$hcst
    obs_fullvalue <- data$obs
    
    # Hindcast climatology 

    data$hcst <- anom$exp
    data$obs <- anom$obs
    remove(anom)
    # Change variable metadata
    # data$hcst$Variable$varName <- paste0(data$hcst$Variable$varName, "anomaly")
    attr(data$hcst$Variable, "variable")$long_name <- 
      paste(attr(data$hcst$Variable, "variable")$long_name, "anomaly")
    # data$obs$Variable$varName <- paste0(data$obs$Variable$varName, "anomaly")
    attr(data$obs$Variable, "variable")$long_name <-
      paste(attr(data$obs$Variable, "variable")$long_name, "anomaly")

    # Compute forecast anomaly field
    if (!is.null(data$fcst)) {
      # Compute hindcast climatology ensemble mean
      clim <- s2dv::Clim(hcst_fullvalue$data, obs_fullvalue$data,
          	         time_dim = "syear",
        	         dat_dim = c("dat", "ensemble"),
        	         memb = FALSE,
        	         memb_dim = "ensemble",
        	         ftime_dim = "time",
			 ncores = recipe$Analysis$ncores)
      clim_hcst <- InsertDim(clim$clim_exp, posdim = 1, lendim = 1,
                             name = "syear")
      dims <- dim(clim_hcst)
      clim_hcst <- rep(clim_hcst, dim(data$fcst$data)[['ensemble']])
      dim(clim_hcst) <- c(dims, ensemble = dim(data$fcst$data)[['ensemble']])
      # Get fcst anomalies
      data$fcst$data <- data$fcst$data - clim_hcst
      # Change metadata
      # data$fcst$Variable$varName <- paste0(data$fcst$Variable$varName, "anomaly")
      attr(data$fcst$Variable, "variable")$long_name <- 
        paste(attr(data$fcst$Variable, "variable")$long_name, "anomaly")
    }

    info(recipe$Run$logger,
  	 paste("The anomalies have been computed,", cross_msg,
	       "cross-validation. The original full fields are returned as",
	       "$hcst.full_val and $obs.full_val."))

    info(recipe$Run$logger, "##### ANOMALIES COMPUTED SUCCESSFULLY #####")

  } else {
    warn(recipe$Run$logger, paste("The Anomalies module has been called, but",
	 "recipe parameter Analysis:Variables:anomaly is set to FALSE.",
	 "The full fields will be returned."))
    hcst_fullvalue <- NULL
    obs_fullvalue <- NULL
    info(recipe$Run$logger, "##### ANOMALIES NOT COMPUTED #####")
  }

  ## TODO: Return fcst full value?

  return(list(hcst = data$hcst, obs = data$obs, fcst = data$fcst,
	      hcst.full_val = hcst_fullvalue, obs.full_val = obs_fullvalue))

}
