#'Read recipe YAML file and create and store logfile info
#'
#'The purpose of this function is to read the recipe configuration for Auto-S2S
#'workflows and create logfiles stores in an the output directory specified in 
#'the recipe. It returns an object of class logger that stores information on 
#'the recipe configuration and errors.
#'
#'@param recipe_file path to a YAML file with Auto-S2S configuration recipe
#'
#'@return list contaning recipe with logger, log file name and log dir name
#'
#'@import log4r
#'@import yaml
#'
#'@examples
#'setwd("/esarchive/scratch/vagudets/repos/auto-s2s/")
#'library(yaml)
#'recipe <- prepare_outputs("modules/data_load/recipe_1.yml")
#'info(recipe$Run$logger, "This is an info message")
#'
#'@export

prepare_outputs <- function(recipe_file) {

# recipe: the content of the readed recipe
# file: the recipe file name

  recipe <- read_yaml(recipe_file)
  recipe$recipe_path <- recipe_file
  recipe$name <- tools::file_path_sans_ext(basename(recipe_file))
  
  output_dir = recipe$Run$output_dir
  # Create output folders:
  folder_name <- paste0(gsub(".yml", "", gsub("/", "_", recipe$name)), "_",
                        gsub(" ", "", gsub(":", "", gsub("-", "", Sys.time()))))

  print("Saving all outputs to:")
  print(output_dir)
  print(folder_name)

  dir.create(file.path(output_dir, folder_name, 'outputs'), recursive = TRUE)
  dir.create(file.path(output_dir, folder_name, 'logs'))
  dir.create(file.path(output_dir, folder_name, 'logs', 'recipes'))

  file.copy(recipe$recipe_path, file.path(output_dir, folder_name, 'logs',
				          'recipes'))

  logfile <- file.path(output_dir, folder_name, 'logs', 'log.txt')
  file.create(logfile)

  # Set default behaviour of log output file:
  if (is.null(recipe$Run)) {
    recipe$Run <- list(Loglevel = 'INFO', Terminal = TRUE)
  }
  if (is.null(recipe$Run$Loglevel)) {
    recipe$Run$Loglevel <- 'INFO'
  }

  if (!is.logical(recipe$Run$Terminal)) {
    recipe$Run$Terminal <- TRUE
  }
  if (recipe$Run$Terminal) {
    logger <- log4r::logger(threshold = recipe$Run$Loglevel,  
                            appenders = list(console_appender(layout = default_log_layout()),
                                             file_appender(logfile, append = TRUE,   
                                                           layout = default_log_layout())))
  } else {
    logger <- log4r::logger(threshold = recipe$Run$Loglevel,
                            appenders = list(file_appender(logfile, append = TRUE,             
                                             layout = default_log_layout())))
  }

  recipe$Run$output_dir <- file.path(output_dir, folder_name)
  recipe$Run$logger <- logger
  recipe$Run$logfile <- logfile

  info(recipe$Run$logger,
       "##### LOGGER SET UP AND OUTPUT DIRECTORY PREPARED #####")

  return(recipe)
}
